import asyncio
import socket
import logging

from asyncio import AbstractEventLoop

from multiprocessing import Process

echo_tasks: list = []


class GracefulExit(SystemExit):
    pass

def shutdown():
    raise GracefulExit()

async def main():
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    # set up the server socket
    server_address = ("127.0.0.1", 8000)
    server_socket.setblocking(False)
    server_socket.bind(server_address)
    server_socket.listen()

    # fires the listening task
    await listen_for_connection(server_socket, asyncio.get_event_loop())


async def listen_for_connection(server_socket: socket, loop: AbstractEventLoop):
    """
    Listen for connections in an infinite loop. Each time receives one creates a task
    ( echo ) that reads and echo data from/to client socket

    """
    while True:
        # blocks until receives a connection
        connection, address = await loop.sock_accept(server_socket)
        # set non-blocking socket mode
        connection.setblocking(False)
        print(f"Got a connection from {address}")
        # schedule an echo task on the event loop for the new socket
        echo_task = asyncio.create_task(echo(connection, loop))
        echo_tasks.append(echo_task)
        print("Alive tasks", len(echo_tasks))


async def echo(connection: socket, loop: AbstractEventLoop) -> None:
    try:
        # blocks until receives data
        while data := await loop.sock_recv(connection, 1024):
            print(f"Got data {data.decode('utf-8')}")
            if data.decode('utf-8') == "shutdown\r\n":
                raise GracefulExit("Shutting down >>>")
            elif data.decode('utf-8') == "crash\r\n":
                raise Exception("Crash sent")
            # sends back data to the connected client
            await loop.sock_sendall(connection, data)
    except Exception as ex_instance:
        logging.exception(ex_instance)
    finally:
        logging.info("Shutting down socket server")
        connection.close()



import asyncio
from multiprocessing import Process
from tests.test_utils import socket_server
import logging


def start_socket_server():
    asyncio.run(socket_server.main())


class TestWorker:

    def test_one_worker(self):
        server_process = Process(target=start_socket_server)
        server_process.start()
        logging.info("Server started")

        # throw worker here

        server_process.join()
        server_process.kill()




import abc
from abc import ABC


class PriceTimeSeries(ABC):

    WEEKDAY_MON: int = 0
    WEEKDAY_TUE: int = 1
    WEEKDAY_WED: int = 2
    WEEKDAY_THU: int = 3
    WEEKDAY_FRI: int = 4
    WEEKDAY_SAT: int = 5
    WEEKDAY_SUN: int = 6

    PRICE_BID: str = "bid"
    PRICE_ASK: str = "ask"
    PRICE_SPREAD: str = "spread"

    @abc.abstractmethod
    def produce(self, *args, **kwargs):
        ...




# import asyncio
# import pytest
# from tests.test_utils import socket_server
# from multiprocessing import Process
#
#
# @pytest.fixture(scope="session")
# def start_socket_server():
#     asyncio.run(socket_server.main())



"""
References

https://numpy.org/doc/stable/reference/random/generated/numpy.random.Generator.uniform.html

"""

from synthetick.price_time_series import PriceTimeSeries
from datetime import datetime
import numpy as np
import pandas as pd


class Ticks(PriceTimeSeries):
    """
    Produce tick time series. This is the core Class on top
    of which other price abstractions are calculated upon.

    Toi generate ticks uses a random walk approach

    # TODO: Produce ticks at random intervals
    # TODO: Improve spread calculation to remove skewed towards 1
    # TODO: Remove weekends
    """

    def __init__(self,
                 trend: float,
                 volatility_range: float,
                 spread_min: float,
                 spread_max: float,
                 pip_position: int,
                 remove_weekend: bool
                 ):
        """

        :param trend: mean for tick distribution in pips
        :param volatility_range: standard deviation for tick distribution in pips
        :param spread_min: minimum spread in pips
        :param spread_max: maximum spread in pips
        :param pip_position: Integer positive or negative indicating the pip change decimal position. If less than
        zero, means the position is at the right of the decimal; point (e.g. 0.1: -1, 0.01: -2, 0.001: -3 etc.). If
        greater than zero, means the pip position is at the right of decimal point (10: 1, 100: 2, etc.). If zero, means
        the pip position is right at the decimal point
        :param remove_weekend: True to remove weekend periods, False otherwise
        """

        self._trend_pip: float = trend
        self._volatility_range_pip: float = volatility_range
        self._spread_min: float = spread_min
        self._spread_max: float = spread_max
        self._pip_position: int = pip_position
        self._remove_weekend: bool = remove_weekend
        self._pip_factor: float = 10 ** pip_position

        self._trend: float | None = None
        self._volatility_range: float | None = None
        self._spread: Spread = Spread(pip_position)
        self.price_time_series: pd.DataFrame | None = None

        self._control_stats: dict = {"total-periods": -1,
                                     "weekend-dropped": -1,
                                     "valid-trading-periods": -1}

        self._validate_parameters()
        self._apply_conversions()

    @property
    def control_stats(self):
        return self._control_stats

    def _validate_parameters(self):
        self._validate_spread_range()
        self._validate_volatility_range()

    def _validate_spread_range(self):
        if self._spread_min <= 0:
            raise ValueError(f"Spread min needs to be grater then zero. {self._spread_min} was provided instead")
        if self._spread_min >= self._spread_max:
            raise ValueError(f"spread_max ({self._spread_max}) needs to be "
                             f"greater than spread_min ({self._spread_min})")

    def _validate_volatility_range(self):
        if self._volatility_range_pip <= 0:
            raise ValueError(f"Volatility range must be positive, got {self._volatility_range_pip} "
                             f"instead")

    def _apply_conversions(self):
        # self._convert_spread_range()
        self._convert_volatility_range()
        self._convert_trend()

    def _convert_volatility_range(self):
        self._volatility_range = self._volatility_range_pip * self._pip_factor

    def _convert_trend(self):
        self._trend = self._trend_pip * self._pip_factor

    def produce(self,
                date_from: datetime = None,
                date_to: datetime = None,
                frequency: str = None,
                init_value: float = None):
        """
        Generates tick data time series between date_from and date_to
        :param date_from: Starting date for the time series
        :param date_to: Limit date for the time series
        :param frequency: Periods frequency. More details here:
            https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases
        :param init_value: Initial value for the time series
        :return:
        """

        if date_from is not None and date_to is not None:
            self._compute_date_range(date_from, date_to, frequency, init_value)
        else:
            raise ValueError("Parameter combination not supported")

        if self._remove_weekend:
            self._remove_weekends()

    def _compute_date_range(self,
                            date_from: datetime,
                            date_to: datetime,
                            frequency: str,
                            init_value: float):
        date_index: pd.DatetimeIndex = pd.date_range(start=date_from,
                                                     end=date_to,
                                                     freq=frequency)
        periods = len(date_index)
        delta_p: np.ndarray = np.random.normal(self._trend, self._volatility_range, periods - 1)
        delta_p = np.append([init_value], delta_p)
        self.price_time_series = pd.DataFrame({"delta_p": delta_p}, index=date_index)
        self.price_time_series[self.PRICE_BID] = self.price_time_series["delta_p"].cumsum()

        self._spread.produce(self._spread_min, self._spread_max, periods)
        self.price_time_series[self.PRICE_SPREAD] = self._spread.spread_raw

        self.price_time_series[self.PRICE_ASK] = self.price_time_series[self.PRICE_BID] + self.price_time_series[
            self.PRICE_SPREAD]

        self._control_stats["total-periods"] = len(self.price_time_series)

    def _remove_weekends(self):

        weekend_filter = (self.price_time_series.index.weekday == self.WEEKDAY_SAT) | (
                self.price_time_series.index.weekday == self.WEEKDAY_SUN)

        self._control_stats["weekends-dropped"] = len(self.price_time_series[weekend_filter])
        self.price_time_series.drop(labels=self.price_time_series[weekend_filter].index, inplace=True)
        self._control_stats["valid-trading-periods"] = len(self.price_time_series)


class Spread:

    def __init__(self, pip_position: int):
        self._pip_factor = 10 ** pip_position
        self._spread: np.ndarray | None = None

    @property
    def spread_raw(self) -> np.ndarray:
        return self._spread * self._pip_factor

    @property
    def spread_pip(self) -> np.ndarray:
        return self._spread

    def produce(self, spread_min: float, spread_max: float, periods: int):
        if spread_min >= spread_max:
            raise ValueError(f"lower_limit ({spread_min}) needs to be less than upper_limit ({spread_max})")
        rng = np.random.default_rng()
        self._spread = rng.uniform(spread_min, spread_max, periods)


class OHLC(PriceTimeSeries):

    def __init__(self,
                 trend: float,
                 volatility_range: float,
                 spread_min: float,
                 spread_max: float,
                 pip_position: int,
                 remove_weekend: bool,
                 tick_frequency: str,
                 time_frame: str):
        """

        :param trend: Set the price time series trend: uptrend, range or downtrend. Its value is the mean of the price
        change for the next period, so if the value is greater and far from zero, the time series produced trends
        upwards. In the other hand, if the value is lower and far from zero, the time series produced trends downwards.
        Finally, if the value is close to zero, the time series produced is trendless or range bound.
        :param volatility_range: Sets the standard deviation for the nex period price change. Indirectly sets the
        price volatility.
        :param spread_min: Minimum spread value in pips
        :param spread_max: maximum spread value in pips
        :param pip_position: At which decimal position the minimum meaningful price change (pip change) occurs.
        :param remove_weekend: If true, remove weekends from produced time series
        :param tick_frequency: At which frequency tick data is produced.
        :param time_frame: OHLC time frame. More details here:
            https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases

        """
        self._trend: float = trend
        self._volatility_range: float = volatility_range
        self._spread_min: float = spread_min
        self._spread_max: float = spread_max
        self._pip_position = pip_position
        self._remove_weekends = remove_weekend
        self._tick_frequency: str = tick_frequency
        self._timeframe: str = time_frame

        self.ohlc_time_series: dict = {self.PRICE_BID: None,
                                       self.PRICE_ASK: None}

    def produce(self,
                date_from: datetime = None,
                date_to: datetime = None,
                init_value: float = None):
        """
        Generates OHLC data time series between date_from and date_to
        :param date_from: Starting date for the time series
        :param date_to: Limit date for the time series
        :param init_value: Initial value for the time series
        :return:
        """

        tick = Ticks(self._trend,
                     self._volatility_range,
                     self._spread_min,
                     self._spread_max,
                     self._pip_position,
                     self._remove_weekends)

        tick.produce(date_from=date_from,
                     date_to=date_to,
                     frequency=self._tick_frequency,
                     init_value=init_value)

        self.ohlc_time_series[self.PRICE_BID] = tick.price_time_series[self.PRICE_BID].resample(self._timeframe).ohlc()
        self.ohlc_time_series[self.PRICE_ASK] = tick.price_time_series[self.PRICE_ASK].resample(self._timeframe).ohlc()

        if self._remove_weekends:
            self.ohlc_time_series[self.PRICE_BID] = self._drop_weekends(self.ohlc_time_series[self.PRICE_BID])
            self.ohlc_time_series[self.PRICE_ASK] = self._drop_weekends(self.ohlc_time_series[self.PRICE_ASK])

    def _drop_weekends(self, price: pd.DataFrame) -> pd.DataFrame:

        weekdays_filter = (price.index.weekday == self.WEEKDAY_SAT) | (
                    price.index.weekday == self.WEEKDAY_SUN)
        weekend_labels = price[weekdays_filter].index
        price.drop(labels=weekend_labels, inplace=True)
        return price





import abc
import logging
from dataclasses import dataclass
from datetime import timedelta, datetime
from time import sleep
import numpy as np


@dataclass
class TickEmissionSpecification:
    instrument: str
    min_change_dec_position: int
    volatility: float
    trend: float
    first_value: float
    tick_frequency_mean: int  # ticks per second
    tick_frequency_std: int  # ticks per second
    spread_min: float
    spread_max: float
    ticks_to_emit: int | None = None
    emit_for_period: timedelta | None = None


class Emitter(abc.ABC):

    def __init__(self, emission_spec: TickEmissionSpecification,
                 dry_run: bool = True):
        self._buffer: list = []
        self._emission_spec = emission_spec
        self._dry_run: bool = dry_run

    @abc.abstractmethod
    def emit(self):
        ...

    @property
    def buffer(self):
        return self._buffer

    def gen_tick(self):

        # generate spread
        rng = np.random.default_rng()
        spread = rng.uniform(self._emission_spec.spread_min,
                             self._emission_spec.spread_max,
                             1).round(self._emission_spec.min_change_dec_position + 1)[0]

        price_change: float = 0

        if len(self._buffer) == 0:
            new_bid = self._emission_spec.first_value
        else:
            price_change = np.random.normal(self._emission_spec.trend,
                                            self._emission_spec.volatility,
                                            1).round(self._emission_spec.min_change_dec_position + 1)[0]
            new_bid = round((self.buffer[-1]["bid"] + price_change), self._emission_spec.min_change_dec_position + 1)

        new_ask = new_bid + spread

        tick = {
            "date-time": datetime.now(),
            "bid": new_bid,
            "ask": new_ask,
            "price-change": price_change,
            "spread": spread
        }

        self._buffer.append(tick)


class TickAmountEmitter(Emitter):

    def __init__(self,
                 emission_spec: TickEmissionSpecification,
                 dry_run: bool = True):

        super().__init__(emission_spec,
                         dry_run=dry_run)

    def emit(self):

        for _ in range(self._emission_spec.ticks_to_emit):
            self.gen_tick()

            if not self._dry_run:
                # send ticks to socket server
                pass

            frequency: float = np.random.normal(self._emission_spec.tick_frequency_mean,
                                                self._emission_spec.tick_frequency_std,
                                                1)[0]
            period: float = 1 / frequency
            sleep(period)


# TODO: Todo next
class TimePeriodEmitter(Emitter):
    def __init__(self, emission_spec: TickEmissionSpecification,
                 dry_run: bool = True):
        super().__init__(emission_spec,
                         dry_run=dry_run)

    def emit(self):
        start_time = datetime.now()
        while datetime.now() - start_time <= self._emission_spec.emit_for_period:
            # TODO: refactor repeated code
            self.gen_tick()
            if not self._dry_run:
                # send tick to socket server
                pass
            frequency: float = np.random.normal(self._emission_spec.tick_frequency_mean,
                                                self._emission_spec.tick_frequency_std,
                                                1)[0]
            period: float = 1 / frequency
            sleep(period)


class EmitterFactory:
    def __call__(self, emission_spec: TickEmissionSpecification,
                 dry_run: bool = True):

        if emission_spec.ticks_to_emit and emission_spec.emit_for_period:
            logging.warning("ticks_to_emit and emmit_for_time were specified, will ticks_to_emit will be assumed")
        if not emission_spec.ticks_to_emit and not emission_spec.emit_for_period:
            logging.error("Neither ticks_to_emit nor emmit_for_period were specified. One MUST be passed")
            raise ValueError("Neither ticks_to_emit nor emmit_for_time were specified. One MUST be passed")

        if emission_spec.ticks_to_emit is not None:
            return TickAmountEmitter(emission_spec,
                                     dry_run=dry_run)

        if emission_spec.emit_for_period is not None:
            return TimePeriodEmitter(emission_spec,
                                     dry_run=dry_run)



from synthetick.synthetick import Spread


class TestSpreadHappyPath:

    SPREAD_LOWER_BOUND_PIP: float = 0.5
    SPREAD_UPPER_BOUND_PIP: float = 3
    PIP_POSITION: float = -4

    def test_basic_path(self):

        spread = Spread(-4)
        spread.produce(self.SPREAD_LOWER_BOUND_PIP, self.SPREAD_UPPER_BOUND_PIP, 1000)
        assert spread.spread_raw.max() <= self.SPREAD_UPPER_BOUND_PIP*10**self.PIP_POSITION
        assert spread.spread_raw.min() >= self.SPREAD_LOWER_BOUND_PIP*10**self.PIP_POSITION






class Worker:

    def __init__(self,
                 instrument: str,
                 volatility: float,
                 trend: float,
                 first_value: float,
                 density_mean: float,
                 density_std: float,
                 ticks_to_emmit: int=1000
                 ):
        pass


    def emit_ticks(self):

        pass








from datetime import datetime
import pandas as pd
from synthetick.synthetick import OHLC

DATE_FROM: datetime = pd.to_datetime("2023-01-01 00:00:00")
DATE_TO: datetime = pd.to_datetime("2023-02-01 00:00:00")


class TestOHLCHappyPath:
    """
    Test OHLC Class
    """

    def test_ohlc_basic(self):
        ohlc: OHLC = OHLC(trend=0.0001,
                          volatility_range=10,
                          spread_min=0.5,
                          spread_max=3,
                          pip_position=-4,
                          remove_weekend=False,
                          tick_frequency="1s",
                          time_frame="H")

        ohlc.produce(date_from=DATE_FROM, date_to=DATE_TO, init_value=1.300)
        assert pd.infer_freq(ohlc.ohlc_time_series["bid"].index) == "H"
        assert ohlc.ohlc_time_series["bid"].index[0] == DATE_FROM
        assert ohlc.ohlc_time_series["bid"].index[-1] == DATE_TO

        ohlc.ohlc_time_series["bid"].to_csv("ohlc_bid_1h.csv", index_label="date-time")

    def test_remove_weekends(self):
        ohlc: OHLC = OHLC(trend=0.0001,
                          volatility_range=10,
                          spread_min=0.5,
                          spread_max=3,
                          pip_position=-4,
                          remove_weekend=True,
                          tick_frequency="1s",
                          time_frame="H")

        ohlc.produce(date_from=DATE_FROM, date_to=DATE_TO, init_value=1.300)
        assert (5 not in ohlc.ohlc_time_series[ohlc.PRICE_ASK].index.weekday) and (6 not in ohlc.ohlc_time_series[ohlc.PRICE_ASK].index.weekday)



from synthetick.price_feed import Worker


class TestWorker:

    def test_worker_basic_path(self):

        instrument = "EURUSD"
        volatility = 4  # pips
        first_value = 1.1300
        frequency_mean = 10  # ticks per second
        frequency_std = 3  # ticks per second
        buffer = 1000   # ticks

        worker = Worker(instrument, volatility, first_value, frequency_mean, frequency_std, buffer=buffer)
        worker.start(for_period=15)

        assert len(worker.buffer) >= 15*(frequency_mean - frequency_std)


class TestServer:

    pass


from datetime import timedelta

import pandas as pd

from synthetick.emitter import EmitterFactory, Emitter, TickEmissionSpecification

FIRST_VALUE: float = 1.13000
TREND: float = 0.0001
VOLATILITY: float = 0.0005
TICKS_TO_EMIT: int = 1000
TICK_FREQUENCY_MEAN: int = 100
TICK_FREQUENCY_STD: int = 10
INSTRUMENT: str = "EURUSD"
SPREAD_MIN: float = 0.0001
SPREAD_MAX: float = 0.0005
MIN_CHANGE_DEC_POSITION: int = 4


class TestEmitter:

    def test_generate_tick_first_tick(self):

        tick_emission_spec = TickEmissionSpecification(instrument=INSTRUMENT,
                                                       volatility=VOLATILITY,
                                                       min_change_dec_position=MIN_CHANGE_DEC_POSITION,
                                                       trend=TREND,
                                                       first_value=FIRST_VALUE,
                                                       tick_frequency_mean=TICK_FREQUENCY_MEAN,
                                                       tick_frequency_std=TICK_FREQUENCY_STD,
                                                       spread_min=SPREAD_MIN,
                                                       spread_max=SPREAD_MAX,
                                                       ticks_to_emit=TICKS_TO_EMIT)
        emitter = EmitterFactory()(tick_emission_spec)
        emitter.gen_tick()
        tick = emitter.buffer[-1]

        assert tick["bid"] == FIRST_VALUE

    def test_generate_tick_two_ticks(self):

        tick_emission_spec = TickEmissionSpecification(instrument=INSTRUMENT,
                                                       volatility=VOLATILITY,
                                                       min_change_dec_position=MIN_CHANGE_DEC_POSITION,
                                                       trend=TREND,
                                                       first_value=FIRST_VALUE,
                                                       tick_frequency_mean=TICK_FREQUENCY_MEAN,
                                                       tick_frequency_std=TICK_FREQUENCY_STD,
                                                       spread_min=SPREAD_MIN,
                                                       spread_max=SPREAD_MAX,
                                                       ticks_to_emit=TICKS_TO_EMIT)
        emitter = EmitterFactory()(tick_emission_spec)
        emitter.gen_tick()
        emitter.gen_tick()
        tick_i = emitter.buffer[-1]
        tick_i_1 = emitter.buffer[-2]

        new_bid = tick_i_1["bid"] + tick_i["price-change"]
        assert tick_i["bid"] == new_bid

    def test_emmit_n_ticks(self):

        tick_emission_spec = TickEmissionSpecification(instrument=INSTRUMENT,
                                                       volatility=VOLATILITY,
                                                       min_change_dec_position=MIN_CHANGE_DEC_POSITION,
                                                       trend=TREND,
                                                       first_value=FIRST_VALUE,
                                                       tick_frequency_mean=TICK_FREQUENCY_MEAN,
                                                       tick_frequency_std=TICK_FREQUENCY_STD,
                                                       spread_min=SPREAD_MIN,
                                                       spread_max=SPREAD_MAX,
                                                       ticks_to_emit=TICKS_TO_EMIT)
        emitter = EmitterFactory()(tick_emission_spec)

        emitter.emit()

        df = pd.DataFrame(emitter.buffer)
        df.to_csv("test_tick_emit_1000.csv")

        assert len(emitter.buffer) == TICKS_TO_EMIT
        assert df["price-change"].mean() - TREND <= 0.0001
        assert df["price-change"].std() - VOLATILITY <= 0.0001

        # TODO: Add check for tick frequency

    def test_emmit_during_time(self):

        emission_period = timedelta(seconds=10)
        tick_emission_spec = TickEmissionSpecification(instrument=INSTRUMENT,
                                                       volatility=VOLATILITY,
                                                       min_change_dec_position=MIN_CHANGE_DEC_POSITION,
                                                       trend=TREND,
                                                       first_value=FIRST_VALUE,
                                                       tick_frequency_mean=TICK_FREQUENCY_MEAN,
                                                       tick_frequency_std=TICK_FREQUENCY_STD,
                                                       spread_min=SPREAD_MIN,
                                                       spread_max=SPREAD_MAX,
                                                       emit_for_period=emission_period)

        emitter = EmitterFactory()(tick_emission_spec)

        emitter.emit()

        df = pd.DataFrame(emitter.buffer)
        df.to_csv("test_tick_emit_20_seconds.csv")

        start_time = emitter.buffer[0]["date-time"]
        end_time = emitter.buffer[-1]["date-time"]
        time_delta = end_time - start_time

        assert time_delta.seconds >= 10
        assert df["bid"].mean() - VOLATILITY <= 0.0001

        # TODO: Add check for tick frequency



# read the contents of your README file
from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory/"README.md").read_text()

setup(
    name='synthetick',
    version='0.1.0',
    packages=['synthetick'],
    url='https://github.com/mchiuminatto/syntetick.git',
    license='',
    author='mchiuminatto',
    author_email='mchiuminatto@gmail.com',
    description='Library to produce synthetic price time series',
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from datetime import datetime
import pandas as pd
from synthetick.synthetick import Ticks

DATE_FROM: datetime = pd.to_datetime("2023-01-01 00:00:00")
DATE_TO: datetime = pd.to_datetime("2023-02-01 00:00:00")


class TestSyntheticHappyPath:

    def test_gen_tick_date_range(self):
        tick_data_generator = Ticks(trend=0.01,
                                    volatility_range=10,
                                    spread_min=0.5,
                                    spread_max=3,
                                    pip_position=-4,
                                    remove_weekend=False)

        tick_data_generator._compute_date_range(date_from=DATE_FROM,
                                                date_to=DATE_TO,
                                                frequency="1s",
                                                init_value=1.1300)
        tick_data_generator.price_time_series.to_csv("test_tick_happy_path.csv", index_label="date-time")

        assert tick_data_generator.price_time_series.index[0] == pd.to_datetime(DATE_FROM)
        assert tick_data_generator.price_time_series.index[-1] == pd.to_datetime(DATE_TO)
        assert pd.infer_freq(tick_data_generator.price_time_series.index) == "S"

    def test_gen_tick_with_compute(self):
        tick_data_generator = Ticks(trend=0.01,
                                    volatility_range=10,
                                    spread_min=0.5,
                                    spread_max=3,
                                    pip_position=-4,
                                    remove_weekend=False)

        tick_data_generator.produce(date_from=DATE_FROM,
                                    date_to=DATE_TO,
                                    frequency="1s",
                                    init_value=1.1300)
        tick_data_generator.price_time_series.to_csv("test_tick_happy_path_with_compute.csv", index_label="date-time")

        assert tick_data_generator.price_time_series.index[0] == pd.to_datetime(DATE_FROM)
        assert tick_data_generator.price_time_series.index[-1] == pd.to_datetime(DATE_TO)
        assert pd.infer_freq(tick_data_generator.price_time_series.index) == "S"

    def test_remove_weekends(self):
        tick_data_generator = Ticks(trend=0.01,
                                    volatility_range=10,
                                    spread_min=0.5,
                                    spread_max=3,
                                    pip_position=-4,
                                    remove_weekend=True)
        tick_data_generator.produce(date_from=DATE_FROM,
                                    date_to=DATE_TO,
                                    frequency="1s",
                                    init_value=1.1300)

        weekday_series: pd.Series = tick_data_generator.price_time_series.index.weekday
        assert (tick_data_generator.WEEKDAY_SAT not in weekday_series and
                tick_data_generator.WEEKDAY_SUN not in weekday_series)
        assert tick_data_generator.control_stats["total-periods"] == tick_data_generator.control_stats[
            "weekends-dropped"] + tick_data_generator.control_stats["valid-trading-periods"]
        tick_data_generator.price_time_series.to_csv("test_tick_happy_path_no_weekend.csv", index_label="date-time")
